@@ -4,6 +4,7 @@ import collections
 import copy
 import datetime
 import hashlib
+import threading
 import time
 import os
 import uuid
@@ -16,6 +17,8 @@ from requests import auth
 from requests import adapters
 from requests.compat import json
 
+import sseclient
+
 ACCESS_TOKEN_URL = 'https://api.home.nest.com/oauth2/access_token'
 AUTHORIZE_URL = 'https://home.nest.com/login/oauth2?client_id={0}&state={1}'
 API_URL = 'https://developer-api.nest.com'
@@ -61,11 +64,21 @@ MAXIMUM_TEMPERATURE_C = 32
 
 
 class APIError(Exception):
-    def __init__(self, response):
-        if response.content != b'':
-            message = response.json()['error']
+    def __init__(self, response, msg=None):
+        try:
+            response_content = response.content
+        except AttributeError:
+            response_content = response.data
+
+        if response_content != b'':
+            if isinstance(response, requests.Response):
+                message = response.json()['error']
         else:
-            message = "Authorization failed"
+            message = "API Error Occured"
+
+        if msg is not None:
+            message = "API Error Occured: " + msg
+
         # Call the base class constructor with the parameters it needs
         super(APIError, self).__init__(message)
 
@@ -73,13 +86,23 @@ class APIError(Exception):
 
 
 class AuthorizationError(Exception):
-    def __init__(self, response):
-        if response.content != b'':
-            message = response.json().get(
-                'error_description',
-                "Authorization Failed")
+    def __init__(self, response, msg=None):
+        try:
+            response_content = response.content
+        except AttributeError:
+            response_content = response.data
+
+        if response_content != b'':
+            if isinstance(response, requests.Response):
+                message = response.json().get(
+                    'error_description',
+                    "Authorization Failed")
         else:
             message = "Authorization failed"
+
+        if msg is not None:
+            message = "Authorization Failed: " + msg
+
         # Call the base class constructor with the parameters it needs
         super(AuthorizationError, self).__init__(message)
 
@@ -121,7 +144,7 @@ class NestAuth(auth.AuthBase):
     def _callback(self, res):
         if self.auth_callback is not None and isinstance(self.auth_callback,
                                                          collections.Callable):
-            self.auth_callback(self._res)
+            self.auth_callback(res)
 
     def login(self, headers=None):
         data = {'client_id': self._client_id,
@@ -166,7 +189,6 @@ class NestBase(object):
         path = '/%s/%s' % (what, self._serial)
 
         response = self._nest_api._put(path=path, data=data)
-        self._nest_api._bust_cache()
 
         return response
 
@@ -1519,7 +1541,7 @@ class Structure(NestBase):
 
 
 class Nest(object):
-    def __init__(self, username=None, password=None, cache_ttl=270,
+    def __init__(self, username=None, password=None,
                  user_agent=None,
                  access_token=None, access_token_cache_file=None,
                  local_time=False,
@@ -1533,9 +1555,10 @@ class Nest(object):
         self._staff = False
         self._superuser = False
         self._email = None
-
-        self._cache_ttl = cache_ttl
-        self._cache = (None, 0)
+        self._queue = collections.deque(maxlen=2)
+        self._event_thread = None
+        self._update_event = threading.Event()
+        self._queue_lock = threading.Lock()
 
         if local_time:
             raise ValueError("local_time no longer supported")
@@ -1543,9 +1566,6 @@ class Nest(object):
         if user_agent:
             raise ValueError("user_agent no longer supported")
 
-        def auth_callback(result):
-            self._access_token = result['access_token']
-
         self._access_token = access_token
         self._client_id = client_id
         self._client_secret = client_secret
@@ -1558,6 +1578,10 @@ class Nest(object):
                         access_token_cache_file=access_token_cache_file)
         self._session.auth = auth
 
+    @property
+    def update_event(self):
+        return self._update_event
+
     @property
     def authorization_required(self):
         return self.never_authorized or \
@@ -1571,7 +1595,7 @@ class Nest(object):
     @property
     def invalid_access_token(self):
         try:
-            self._status
+            self._get("/")
             return False
         except AuthorizationError:
             return True
@@ -1579,7 +1603,6 @@ class Nest(object):
     @property
     def client_version_out_of_date(self):
         if self._product_version is not None:
-            self._bust_cache()
             try:
                 return self.client_version < self._product_version
             # an error means they need to authorize anyways
@@ -1600,6 +1623,99 @@ class Nest(object):
     def access_token(self):
         return self._access_token or self._session.auth.access_token
 
+    def _handle_ratelimit(self, res, verb, url, data,
+                          max_retries=10, default_wait=5,
+                          stream=False, headers=None):
+        response = res
+        retries = 0
+        while response.status_code == 429 and retries <= max_retries:
+            retries += 1
+            retry_after = response.headers['Retry-After']
+            # Default Retry Time
+            wait = default_wait
+
+            try:
+                # Checks if retry_after is a number
+                wait = float(retry_after)
+            except ValueError:
+                # If not:
+                try:
+                    # Checks if retry_after is a HTTP date
+                    now = datetime.datetime.now()
+                    wait = (now - parse_time(retry_after)).total_seconds()
+                except ValueError:
+                    # Does nothing and uses default (shouldn't happen)
+                    pass
+
+            time.sleep(wait)
+            response = self._session.request(verb, url,
+                                             allow_redirects=False,
+                                             stream=stream,
+                                             headers=headers,
+                                             data=data)
+        return response
+
+    def _open_data_stream(self, path="/"):
+        url = "%s%s" % (API_URL, path)
+
+        # Opens the data stream
+        headers = {'Accept': 'text/event-stream'}
+        response = self._session.get(url, stream=True, headers=headers,
+                                     allow_redirects=False)
+
+        if response.status_code == 401:
+            raise AuthorizationError(response)
+
+        if response.status_code == 429:
+            response = self._handle_ratelimit(response, 'GET', url, None,
+                                              max_retries=10,
+                                              default_wait=5,
+                                              stream=True,
+                                              headers=headers)
+
+        if response.status_code == 307:
+            redirect_url = response.headers['Location']
+            response = self._session.get(redirect_url,
+                                         allow_redirects=False,
+                                         headers=headers,
+                                         stream=True)
+            if response.status_code == 429:
+                response = self._handle_ratelimit(response, 'GET', url, None,
+                                                  max_retries=10,
+                                                  default_wait=5,
+                                                  stream=True,
+                                                  headers=headers)
+
+        ready_event = threading.Event()
+        self._event_thread = threading.Thread(target=self._start_event_loop,
+                                              args=(response,
+                                                    self._queue,
+                                                    ready_event,
+                                                    self._update_event))
+        self._event_thread.setDaemon(True)
+        self._event_thread.start()
+        ready_event.wait(timeout=10)
+
+    def _start_event_loop(self, response, queue, ready_event, update_event):
+        client = sseclient.SSEClient(response.iter_content())
+        for event in client.events():
+            event_type = event.event
+            if event_type == 'open' or event_type == 'keep-alive':
+                pass
+            elif event_type == 'put':
+                queue.appendleft(json.loads(event.data))
+                update_event.set()
+            elif event_type == 'auth_revoked':
+                raise AuthorizationError(None,
+                                         msg='Auth token has been revoked')
+            elif event_type == 'error':
+                raise APIError(None, msg=event.data)
+
+            if not ready_event.is_set():
+                ready_event.set()
+        response.close()
+        queue.clear()
+
     def _request(self, verb, path="/", data=None):
         url = "%s%s" % (API_URL, path)
 
@@ -1615,6 +1731,17 @@ class Nest(object):
         if response.status_code == 401:
             raise AuthorizationError(response)
 
+        # Rate Limit Exceeded Catch
+        if response.status_code == 429:
+            response = self._handle_ratelimit(response, verb, url, data,
+                                              max_retries=10,
+                                              default_wait=5)
+
+            # Prevent this from catching as APIError
+            if response.status_code == 200:
+                return response.json()
+
+        # This will handle the error if max_retries is exceeded
         if response.status_code != 307:
             raise APIError(response)
 
@@ -1622,8 +1749,14 @@ class Nest(object):
         response = self._session.request(verb, redirect_url,
                                          allow_redirects=False,
                                          data=data)
-        # TODO check for 429 status code for too frequent access.
-        # see https://developers.nest.com/documentation/cloud/data-rate-limits
+
+        # Rate Limit Exceeded Catch
+        if response.status_code == 429:
+            response = self._handle_ratelimit(response, verb, redirect_url,
+                                              data, max_retries=10,
+                                              default_wait=5)
+
+        # This will handle the error if max_retries is exceeded
         if 400 <= response.status_code < 600:
             raise APIError(response)
 
@@ -1643,12 +1776,16 @@ class Nest(object):
 
     @property
     def _status(self):
-        value, last_update = self._cache
-        now = time.time()
+        self._queue_lock.acquire()
+        if len(self._queue) == 0 or not self._queue[0]:
+            self._open_data_stream("/")
+        self._queue_lock.release()
 
-        if not value or now - last_update > self._cache_ttl:
+        self._queue_lock.acquire(False)
+        value = self._queue[0]['data']
+        self._queue_lock.release()
+        if not value:
             value = self._get("/")
-            self._cache = (value, now)
 
         return value
 
@@ -1664,9 +1801,6 @@ class Nest(object):
     def _devices(self):
         return self._status[DEVICES]
 
-    def _bust_cache(self):
-        self._cache = (None, 0)
-
     @property
     def devices(self):
         raise NotImplementedError("Use thermostats instead")
